@@ -4,7 +4,7 @@ from satori.model import (
     Channel,
     Event,
     Guild,
-    Login,
+    LoginPartial,
     Member,
     MessageObject,
     Role,
@@ -27,6 +27,10 @@ class GuildEvent(Event):
     guild: Guild
 
 
+class ChannelEvent(GuildEvent):
+    channel: Channel
+
+
 class GuildMemberEvent(GuildEvent):
     user: User
     member: Member
@@ -37,7 +41,7 @@ class GuildRoleEvent(GuildEvent):
 
 
 class LoginEvent(Event):
-    login: Login
+    login: LoginPartial
 
 
 class ReactionEvent(Event):
