@@ -1,11 +1,12 @@
 import mimetypes
-from dataclasses import asdict, dataclass, field, fields
+from collections.abc import AsyncIterable, Awaitable, Callable
+from dataclasses import Field, dataclass, field
 from datetime import datetime
 from enum import IntEnum
 from os import PathLike
 from pathlib import Path
-from typing import IO, Any, Callable, ClassVar, Generic, Literal, Optional, TypeVar, Union
-from typing_extensions import TypeAlias
+from typing import IO, Any, ClassVar, Generic, Literal, TypeAlias, TypeVar
+from typing_extensions import Self
 
 from .element import Element, transform
 from .parser import Element as RawElement
@@ -15,18 +16,21 @@ from .parser import parse
 @dataclass
 class ModelBase:
     __converter__: ClassVar[dict[str, Callable[[Any], Any]]] = {}
+    _raw_data: dict[str, Any] = field(init=False, default_factory=dict, repr=False, compare=False, hash=False)
 
     @classmethod
-    def parse(cls, raw: dict):
-        fs = fields(cls)
+    def parse(cls: type[Self], raw: dict) -> Self:
+        fs: dict[str, Field] = cls.__dataclass_fields__
         data = {}
-        for fd in fs:
-            if fd.name in raw:
-                if fd.name in cls.__converter__:
-                    data[fd.name] = cls.__converter__[fd.name](raw[fd.name])
+        for name in fs.keys():
+            if name in raw:
+                if name in cls.__converter__:
+                    data[name] = cls.__converter__[name](raw[name])
                 else:
-                    data[fd.name] = raw[fd.name]
-        return cls(**data)  # type: ignore
+                    data[name] = raw[name]
+        obj = cls(**data)  # type: ignore
+        obj._raw_data = raw
+        return obj
 
     def dump(self) -> dict:
         raise NotImplementedError
@@ -42,9 +46,9 @@ class ChannelType(IntEnum):
 @dataclass
 class Channel(ModelBase):
     id: str
-    type: ChannelType
-    name: Optional[str] = None
-    parent_id: Optional[str] = None
+    type: ChannelType = ChannelType.TEXT
+    name: str | None = None
+    parent_id: str | None = None
 
     __converter__ = {"type": ChannelType}
 
@@ -60,8 +64,8 @@ class Channel(ModelBase):
 @dataclass
 class Guild(ModelBase):
     id: str
-    name: Optional[str] = None
-    avatar: Optional[str] = None
+    name: str | None = None
+    avatar: str | None = None
 
     def dump(self):
         res = {"id": self.id}
@@ -75,10 +79,10 @@ class Guild(ModelBase):
 @dataclass
 class User(ModelBase):
     id: str
-    name: Optional[str] = None
-    nick: Optional[str] = None
-    avatar: Optional[str] = None
-    is_bot: Optional[bool] = None
+    name: str | None = None
+    nick: str | None = None
+    avatar: str | None = None
+    is_bot: bool | None = None
 
     def dump(self):
         res: dict[str, Any] = {"id": self.id}
@@ -95,10 +99,10 @@ class User(ModelBase):
 
 @dataclass
 class Member(ModelBase):
-    user: Optional[User] = None
-    nick: Optional[str] = None
-    avatar: Optional[str] = None
-    joined_at: Optional[datetime] = None
+    user: User | None = None
+    nick: str | None = None
+    avatar: str | None = None
+    joined_at: datetime | None = None
 
     __converter__ = {"user": User.parse, "joined_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000)}
 
@@ -118,7 +122,7 @@ class Member(ModelBase):
 @dataclass
 class Role(ModelBase):
     id: str
-    name: Optional[str] = None
+    name: str | None = None
 
     def dump(self):
         res = {"id": self.id}
@@ -129,69 +133,63 @@ class Role(ModelBase):
 
 class LoginStatus(IntEnum):
     OFFLINE = 0
+    """离线"""
     ONLINE = 1
+    """在线"""
     CONNECT = 2
+    """正在连接"""
     DISCONNECT = 3
+    """正在断开连接"""
     RECONNECT = 4
+    """正在重新连接"""
 
 
 @dataclass
 class Login(ModelBase):
+    sn: int
     status: LoginStatus
-    user: Optional[User] = None
-    self_id: Optional[str] = None
-    platform: Optional[str] = None
+    adapter: str
+    platform: str
+    user: User
     features: list[str] = field(default_factory=list)
-    proxy_urls: list[str] = field(default_factory=list)
 
     __converter__ = {"user": User.parse, "status": LoginStatus}
 
     def dump(self):
         res: dict[str, Any] = {
+            "sn": self.sn,
             "status": self.status.value,
-            "features": self.features,
-            "proxy_urls": self.proxy_urls,
+            "adapter": self.adapter,
         }
-        if self.user:
-            res["user"] = self.user.dump()
-        if self.self_id:
-            res["self_id"] = self.self_id
         if self.platform:
             res["platform"] = self.platform
+        if self.user:
+            res["user"] = self.user.dump()
+        if self.features:
+            res["features"] = self.features
         return res
 
-    @property
-    def id(self) -> Optional[str]:
-        return self.self_id or (self.user.id if self.user else None)
-
-
-@dataclass
-class LoginPreview(ModelBase):
-    user: User
-    platform: str
-    status: Optional[LoginStatus] = None
-    features: list[str] = field(default_factory=list)
-    proxy_urls: list[str] = field(default_factory=list)
-
-    __converter__ = {"user": User.parse, "status": LoginStatus}
-
-    def dump(self):
-        res: dict[str, Any] = {
-            "user": self.user.dump(),
-            "platform": self.platform,
-            "features": self.features,
-            "proxy_urls": self.proxy_urls,
-        }
-        if self.status:
-            res["status"] = self.status.value
-        return res
+    @classmethod
+    def parse(cls, raw: dict):
+        if "self_id" in raw and "user" not in raw:
+            raw["user"] = {"id": raw["self_id"]}
+        if "sn" not in raw:
+            raw["sn"] = 0
+        if "adapter" not in raw:
+            raw["adapter"] = "satori"
+        if "status" not in raw:
+            raw["status"] = LoginStatus.ONLINE
+        return super().parse(raw)
 
     @property
     def id(self) -> str:
         return self.user.id
 
 
-LoginType = Union[Login, LoginPreview]
+@dataclass
+class LoginPartial(Login):
+    platform: str | None = None
+    user: User | None = None
 
 
 @dataclass
@@ -201,64 +199,129 @@ class ArgvInteraction(ModelBase):
     options: Any
 
     def dump(self):
-        return asdict(self)
+        return {"name": self.name, "arguments": self.arguments, "options": self.options}
 
 
 @dataclass
 class ButtonInteraction(ModelBase):
     id: str
+    data: str | None = None
 
     def dump(self):
-        return asdict(self)
+        res = {"id": self.id}
+        if self.data:
+            res["data"] = self.data
+        return res
 
 
 class Opcode(IntEnum):
     EVENT = 0
+    """事件 (接收)"""
     PING = 1
+    """心跳 (发送)"""
     PONG = 2
+    """心跳回复 (接收)"""
     IDENTIFY = 3
+    """鉴权 (发送)"""
     READY = 4
+    """鉴权成功 (接收)"""
+    META = 5
+    """元信息更新 (接收)"""
 
 
 @dataclass
 class Identify(ModelBase):
-    token: Optional[str] = None
-    sequence: Optional[int] = None
+    token: str | None = None
+    sn: int | None = None
+
+    @classmethod
+    def parse(cls, raw: dict):
+        if "sequence" in raw and "sn" not in raw:
+            raw["sn"] = raw["sequence"]
+        return super().parse(raw)
+
+    @property
+    def sequence(self) -> int | None:
+        return self.sn
+
+    def dump(self):
+        return {k: v for k, v in (("token", self.token), ("sn", self.sn)) if v is not None}
 
 
 @dataclass
 class Ready(ModelBase):
-    logins: list[Login]
+    logins: list[LoginPartial]
+    proxy_urls: list[str] = field(default_factory=list)
+
+    __converter__ = {"logins": lambda raw: [LoginPartial.parse(login) for login in raw]}
+
+    def dump(self):
+        return {"logins": [login.dump() for login in self.logins], "proxy_urls": self.proxy_urls}
+
+
+@dataclass
+class MetaPayload(ModelBase):
+    """Meta 信令"""
+
+    proxy_urls: list[str]
+
+    def dump(self):
+        return {"proxy_urls": self.proxy_urls}
+
+
+@dataclass
+class Meta(ModelBase):
+    """Meta 数据"""
+
+    logins: list[LoginPartial]
+    proxy_urls: list[str] = field(default_factory=list)
+
+    __converter__ = {"logins": lambda raw: [LoginPartial.parse(login) for login in raw]}
+
+    def dump(self):
+        return {"logins": [login.dump() for login in self.logins], "proxy_urls": self.proxy_urls}
 
 
 @dataclass
 class MessageObject(ModelBase):
     id: str
-    content: str
-    channel: Optional[Channel] = None
-    guild: Optional[Guild] = None
-    member: Optional[Member] = None
-    user: Optional[User] = None
-    created_at: Optional[datetime] = None
-    updated_at: Optional[datetime] = None
+    content: str = ""
+    channel: Channel | None = None
+    guild: Guild | None = None
+    member: Member | None = None
+    user: User | None = None
+    created_at: datetime | None = None
+    updated_at: datetime | None = None
+    referrer: dict | None = None
 
     @classmethod
     def from_elements(
         cls,
         id: str,
         content: list[Element],
-        channel: Optional[Channel] = None,
-        guild: Optional[Guild] = None,
-        member: Optional[Member] = None,
-        user: Optional[User] = None,
-        created_at: Optional[datetime] = None,
-        updated_at: Optional[datetime] = None,
+        channel: Channel | None = None,
+        guild: Guild | None = None,
+        member: Member | None = None,
+        user: User | None = None,
+        created_at: datetime | None = None,
+        updated_at: datetime | None = None,
+        referrer: dict | None = None,
     ):
-        return cls(id, "".join(str(i) for i in content), channel, guild, member, user, created_at, updated_at)
+        obj = cls(id, "".join(str(i) for i in content), channel, guild, member, user, created_at, updated_at, referrer)
+        obj._parsed_message = content
+        return obj
 
     @property
     def message(self) -> list[Element]:
-        return transform(parse(self.content))
+        if hasattr(self, "_parsed_message"):
+            return self._parsed_message
+        self._parsed_message = transform(parse(self.content))
+        return self._parsed_message
+
+    @message.setter
+    def message(self, value: list[Element]):
+        self._parsed_message = value
+        self.content = "".join(str(i) for i in value)
 
     @classmethod
     def parse(cls, raw: dict):
@@ -290,60 +353,31 @@ class MessageObject(ModelBase):
             res["created_at"] = int(self.created_at.timestamp() * 1000)
         if self.updated_at:
             res["updated_at"] = int(self.updated_at.timestamp() * 1000)
-        return res
-
-
-@dataclass
-class MessageReceipt(ModelBase):
-    id: str
-    content: Optional[str] = None
-
-    @classmethod
-    def from_elements(
-        cls,
-        id: str,
-        content: Optional[list[Element]] = None,
-    ):
-        return cls(id, "".join(str(i) for i in content) if content else None)
-
-    @property
-    def message(self) -> Optional[list[Element]]:
-        return transform(parse(self.content)) if self.content else None
-
-    @classmethod
-    def parse(cls, raw: dict):
-        if "elements" in raw and "content" not in raw:
-            content = [RawElement(*item.values()) for item in raw["elements"]]
-            raw["content"] = "".join(str(i) for i in content)
-        return super().parse(raw)
-
-    def dump(self):
-        res = {"id": self.id}
-        if self.content:
-            res["content"] = self.content
+        if self.referrer:
+            res["referrer"] = self.referrer
         return res
 
 
 @dataclass
 class Event(ModelBase):
-    id: int
     type: str
     timestamp: datetime
-    platform: Optional[str] = None
-    self_id: Optional[str] = None
-    argv: Optional[ArgvInteraction] = None
-    button: Optional[ButtonInteraction] = None
-    channel: Optional[Channel] = None
-    guild: Optional[Guild] = None
-    login: Optional[LoginType] = None
-    member: Optional[Member] = None
-    message: Optional[MessageObject] = None
-    operator: Optional[User] = None
-    role: Optional[Role] = None
-    user: Optional[User] = None
+    login: Login
+    argv: ArgvInteraction | None = None
+    button: ButtonInteraction | None = None
+    channel: Channel | None = None
+    guild: Guild | None = None
+    member: Member | None = None
+    message: MessageObject | None = None
+    operator: User | None = None
+    role: Role | None = None
+    user: User | None = None
+    referrer: dict | None = None
 
-    _type: Optional[str] = None
-    _data: Optional[dict] = None
+    _type: str | None = None
+    _data: dict | None = None
+
+    sn: int = 0
 
     __converter__ = {
         "timestamp": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
@@ -351,13 +385,7 @@ class Event(ModelBase):
         "button": ButtonInteraction.parse,
         "channel": Channel.parse,
         "guild": Guild.parse,
-        "login": lambda raw: (
-            LoginPreview.parse(
-                raw if raw["user"] else {**raw, "user": {"id": raw["self_id"]}} if "self_id" in raw else raw
-            )
-            if "user" in raw
-            else Login.parse(raw)
-        ),
+        "login": Login.parse,
         "member": Member.parse,
         "message": MessageObject.parse,
         "operator": User.parse,
@@ -365,33 +393,37 @@ class Event(ModelBase):
         "user": User.parse,
     }
 
-    @property
-    def platform_(self):
-        if self.platform:
-            return self.platform
-        if self.login and self.login.platform:
-            return self.login.platform
-        raise ValueError("platform not found")
+    @classmethod
+    def parse(cls, raw: dict):
+        if "id" in raw and "sn" not in raw:
+            raw["sn"] = raw["id"]
+        if "platform" in raw and "self_id" in raw and "login" not in raw:
+            raw["login"] = {
+                "sn": 0,
+                "platform": raw["platform"],
+                "user": {"id": raw["self_id"]},
+                "status": LoginStatus.ONLINE,
+            }
+        if "self_id" in raw and not raw.get("login", {}).get("user"):
+            if "login" not in raw:
+                raw["login"] = {"sn": 0, "status": LoginStatus.ONLINE, "platform": raw.get("platform", "unknown")}
+            raw["login"]["user"] = {"id": raw["self_id"]}
+        return super().parse(raw)
 
     @property
-    def self_id_(self):
-        if self.self_id:
-            return self.self_id
-        if self.login and self.login.id:
-            return self.login.id
-        raise ValueError("self_id not found")
+    def platform(self):
+        return self.login.platform
 
-    def __post_init__(self):
-        _ = self.platform_
-        _ = self.self_id_
+    @property
+    def self_id(self):
+        return self.login.id
 
     def dump(self):
         res = {
-            "id": self.id,
+            "sn": self.sn,
             "type": self.type,
-            "platform": self.platform_,
-            "self_id": self.self_id_,
             "timestamp": int(self.timestamp.timestamp() * 1000),
+            "login": self.login.dump(),
         }
         if self.argv:
             res["argv"] = self.argv.dump()
@@ -413,6 +445,8 @@ class Event(ModelBase):
             res["role"] = self.role.dump()
         if self.user:
             res["user"] = self.user.dump()
+        if self.referrer:
+            res["referrer"] = self.referrer
         if self._type:
             res["_type"] = self._type
         if self._data:
@@ -426,10 +460,10 @@ T = TypeVar("T", bound=ModelBase)
 @dataclass
 class PageResult(ModelBase, Generic[T]):
     data: list[T]
-    next: Optional[str] = None
+    next: str | None = None
 
     @classmethod
-    def parse(cls, raw: dict, parser: Optional[Callable[[dict], T]] = None) -> "PageResult[T]":
+    def parse(cls, raw: dict, parser: Callable[[dict], T] | None = None) -> "PageResult[T]":
         data = [(parser or ModelBase.parse)(item) for item in raw["data"]]
         return cls(data, raw.get("next"))  # type: ignore
 
@@ -442,10 +476,10 @@ class PageResult(ModelBase, Generic[T]):
 
 @dataclass
 class PageDequeResult(PageResult[T]):
-    prev: Optional[str] = None
+    prev: str | None = None
 
     @classmethod
-    def parse(cls, raw: dict, parser: Optional[Callable[[dict], T]] = None) -> "PageDequeResult[T]":
+    def parse(cls, raw: dict, parser: Callable[[dict], T] | None = None) -> "PageDequeResult[T]":
         data = [(parser or ModelBase.parse)(item) for item in raw["data"]]
         return cls(data, raw.get("next"), raw.get("prev"))  # type: ignore
 
@@ -458,15 +492,36 @@ class PageDequeResult(PageResult[T]):
         return res
 
 
+class IterablePageResult(Generic[T], AsyncIterable[T], Awaitable[PageResult[T]]):
+    def __init__(self, func: Callable[[str | None], Awaitable[PageResult[T]]], initial_page: str | None = None):
+        self.func = func
+        self.next_page = initial_page
+
+    def __await__(self):
+        return self.func(self.next_page).__await__()
+
+    def __aiter__(self):
+        async def _gen():
+            while True:
+                result = await self.func(self.next_page)
+                for item in result.data:
+                    yield item
+                self.next_page = result.next
+                if not self.next_page:
+                    break
+
+        return _gen()
+
+
 Direction: TypeAlias = Literal["before", "after", "around"]
 Order: TypeAlias = Literal["asc", "desc"]
 
 
 @dataclass
 class Upload:
-    file: Union[bytes, IO[bytes], PathLike]
+    file: bytes | IO[bytes] | PathLike
     mimetype: str = "image/png"
-    name: Optional[str] = None
+    name: str | None = None
 
     def __post_init__(self):
         if isinstance(self.file, PathLike):
